/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_LANEBOUNDARY_H
#define DPC_IMPL_TYPE_LANEBOUNDARY_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint32.h"
#include "dpc/impl_type_linegeometry.h"
#include "impl_type_float.h"
#include "dpc/impl_type_vector_samplingpoint.h"

namespace dpc {
struct LaneBoundary {
    ::UInt32 line_boundary_id{ 0U };
    ::dpc::LineGeometry line_geometry;
    ::Float arc_length{ 0.0F };
    ::dpc::vector_SamplingPoint sampling_point_list;

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(line_boundary_id);
        fun(line_geometry);
        fun(arc_length);
        fun(sampling_point_list);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(line_boundary_id);
        fun(line_geometry);
        fun(arc_length);
        fun(sampling_point_list);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("line_boundary_id", line_boundary_id);
        fun("line_geometry", line_geometry);
        fun("arc_length", arc_length);
        fun("sampling_point_list", sampling_point_list);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("line_boundary_id", line_boundary_id);
        fun("line_geometry", line_geometry);
        fun("arc_length", arc_length);
        fun("sampling_point_list", sampling_point_list);
    }

    friend bool operator==(const ::dpc::LaneBoundary& lhs, const ::dpc::LaneBoundary& rhs) noexcept
    {
        return (lhs.line_boundary_id == rhs.line_boundary_id) && (lhs.line_geometry == rhs.line_geometry) && (fabs(static_cast<double>(lhs.arc_length - rhs.arc_length)) < DBL_EPSILON) && (lhs.sampling_point_list == rhs.sampling_point_list);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_LANEBOUNDARY_H
